/* inverse_thomas
 *  Copyright (C) 2000 Luc Brun
 *  email: brun@leri.univ-reims.fr
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the version 2 of the GNU General Public
 *  License as published by the Free Software Foundation.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/
#include <stdio.h>
#include <image.h>
#include <point.h>
#include <chrono.h>
#include "distance.h"
#include "types.h"

image inverse_colormap(image img_source,
		       image table)
{
  cubeRGB classe,distance;
  delaunay del;
  int* table_bis;
  int *pixel_table,*pixel_source,*pixel_dest;
  image img_dest;
  Point p;
  int indice;
  int nb_couleur,i,ind;
  int R,G,B;

  nb_couleur = image_give_largeur(table);
  table_bis = (int*) malloc(image_give_dim(table)*sizeof(int)*nb_couleur);
  image_debut(table);
  ind=0;
    do
      {
	pixel_table = image_lire_pixel(table);
	for(i=0;i<3;i++)
	  {
	    table_bis[ind] = pixel_table[i];
	    ind++;
	  }
      }
    while(image_pixel_suivant(table));
	/* Initialisation du cube. */
  image_dist(table_bis,nb_couleur,classe,distance,del);      
  COORDY(p) =0;
  img_dest = FAIRE_image();
  image_initialize(img_dest,
		   image_give_dim(img_source),
		   image_give_largeur(img_source),
		   image_give_hauteur(img_source)
		   );

  free(table_bis);

  image_debut(img_source);
  image_debut(img_dest);
  do
    {
      pixel_source = image_lire_pixel(img_source);
      R = pixel_source[0];
      G = pixel_source[1];
      B = pixel_source[2];
      

      indice = classe[R>>K_REDUCTION][G>>K_REDUCTION][B>>K_REDUCTION];
      
      COORDX(p) = indice;
      assert(image_move_to(table,&p) != 0);
      pixel_dest = image_lire_pixel(table);
      image_ecrire_pixel(img_dest,pixel_dest);
      image_pixel_suivant(img_dest);
    }
  while(image_pixel_suivant(img_source));

  return img_dest;
}
void usage(char * nom)
{
  fprintf(stderr,"USAGE : %s file.ppm tab.ppm > res.ppm\n",nom);
}
main(int argc,char *argv[])
{
  image img_source,table,img_dest;
  long temps;

  if(argc != 3)
    {
      usage(argv[0]);
      exit(1);
    }
  img_source = FAIRE_image();
  if(image_charger(img_source,argv[1]) == -1)
    {
      fprintf(stderr,"Ouverture fichier %s impossible...\n Exit\n",argv[1]);
      exit(2);
    }
	
  table = FAIRE_image();
  if(image_charger(table,argv[2])==-1)
    {
      fprintf(stderr,"Ouverture fichier %s impossible...\n Exit\n",argv[1]);
      exit(2);
    }
  
  temps = chrono(img_dest = inverse_colormap(img_source,table));
  fprintf(stderr,"Temps total :%f\n", ((float)temps)/CLK_TCK);
  image_to_stream(img_dest,stdout);


  DEFAIRE_image(img_dest);
  DEFAIRE_image(img_source);
  DEFAIRE_image(table);
}
